<?php

namespace App\HttpController\Google;

use App\Key\K;
use App\Tools\System;
use EasySwoole\Component\TableManager;
use Swlib\Saber;

class Proxy extends Base
{
    public function index()
    {
        $subServer = \App\Tools\Proxy::subServer();
        if (!$subServer) $subServer = [];
        $originRequest = $this->request()->getSwooleRequest();
        $method = $this->request()->getMethod();
        $path = strtolower($originRequest->server['request_uri']);
        //把默认本机的地址返回
        $systemConfig=System::config();
        $serverPort=$systemConfig[K::KEY_SERVER_PORT];
        if (!$serverPort) $serverPort=9502;
        if ($systemConfig[K::KEY_GOOGLE_HISTORY_ENABLE]) $subServer[] = ['host' => '127.0.0.1', 'port' =>$serverPort];
        if (!count($subServer)) {
            $this->setStatusCode(500);
            return;
        }
        if (in_array(trim($systemConfig[K::KEY_SERVER_ID]),K::CONST_INNER_SERVER_ID)){
            //如果是内网中的主控制服务器，那么要按时间来进行分批处理服务器
            if (count($subServer)>1){
                $delta=5;
//            $currentSleep=floor((time()/(35*60)))%$delta;
                $currentIndex=TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG)->get(K::KEY_SLEEP_INDEX,'number');
                $currentIndex=intval($currentIndex);
                if (!is_numeric($currentIndex)||$currentIndex<0) $currentIndex=0;
                $currentSleep=$currentIndex%$delta;
                $deltaSize=ceil(count($subServer)/$delta);
                array_splice($subServer,$currentSleep*$deltaSize,$deltaSize);
            }
        }
        if ($path=='/flatfile'){
            $tileSum=TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG)->incr(K::KEY_TILE_GET_SUM,'number');
            $subServer=array_values($subServer);
            $whichIndex=$tileSum%count($subServer);
            $selectProxy = $subServer[$whichIndex];
        }else{
            shuffle($subServer);
            $selectProxy = current($subServer);
        }
//        echo $selectProxy['host'].':'.$selectProxy['port'],PHP_EOL;
        $header = $this->header();
        $header['bigemapip'] = $this->ip;
        $request = Saber::create([
            'base_uri' => $selectProxy['host'] . ':' . $selectProxy['port'],
            'use_pool' => true,
        ]);
        $cookies = $this->getCookie();
        switch ($path) {
            case '/dbroot':
            case '/dbroot.v5':
                if ($method !== 'GET') return;
//            strpos($selectProxy['host'], '127.0.0.1') !== false
                if ($selectProxy['port']==$serverPort) {
                    $url = '/google/index/db?' . $originRequest->server['query_string'];
                } else {
                    $url = '/dbRoot?' . $originRequest->server['query_string'];
                }
                try {
                    $data = $request->get($url, ['timeout' => 3, 'headers' => $header, 'cookies' => $cookies]);
                    $responseHeaders = $data->getHeaders();
//                    System::log($responseHeaders, 'proxy response header');
                    if ($responseHeaders['server-uu']) {
                        $this->header('server-uu', current($responseHeaders['server-uu']));
                    }
                    $this->header('content-type', 'application/octet-stream');
                    $this->response()->getSwooleResponse()->write($data->getBody()->getContents());
                } catch (\Throwable $exception) {
                    $errorCode = $exception->getCode();
                    switch ($errorCode) {
                        case 403:
                            $this->setStatusCode(403);
                            $this->response()->getSwooleResponse()->write($exception->getMessage());
                            break;
                        case 404:
                            $this->setStatusCode(404);
                            $this->response()->getSwooleResponse()->write($exception->getMessage());
                            break;
                        default:
                            $this->setStatusCode(500);
                            $this->header('cc', $exception->getCode());
                            $this->response()->getSwooleResponse()->write($exception->getMessage());
                    }
                }
                break;
            case '/flatfile':
                if ($method !== 'GET') return;
                if ($selectProxy['port']==$serverPort) {
                    $url = '/google/index/tile?' . $originRequest->server['query_string'];
                } else {
                    $url = '/flatfile?' . $originRequest->server['query_string'];
                }
                try {
                    $data = $request->get($url, ['timeout' => 3, 'headers' => $header, 'cookies' => $cookies]);
                    $responseHeaders = $data->getHeaders();
                    if ($responseHeaders['server-uu']) {
                        $this->header('server-uu', current($responseHeaders['server-uu']) . 'p');
                    }
                    $this->header('content-type', 'application/octet-stream');
                    $this->response()->getSwooleResponse()->write($data->getBody()->getContents());
                } catch (\Throwable $exception) {
                    $errorCode = $exception->getCode();
//                    echo $exception->getCode(), '====', $exception->getMessage(), PHP_EOL;
                    switch ($errorCode) {
                        case 403:
                            $this->setStatusCode(403);
                            $this->response()->getSwooleResponse()->write($exception->getMessage());
                            break;
                        case 404:
                            $this->setStatusCode(404);
                            $this->response()->getSwooleResponse()->write($exception->getMessage());
                            break;
                        default:
                            $this->setStatusCode(500);
                            $this->header('cc', $exception->getCode());
                            $this->response()->getSwooleResponse()->write($exception->getMessage());
                    }
                }
                break;
            case '/geauth':
                if ($method !== 'POST') return;
                if ($selectProxy['port']==$serverPort) {
                    $url = '/google/index/auth?' . $originRequest->server['query_string'];
                } else {
                    $url = '/geauth?' . $originRequest->server['query_string'];
                }
                try {
                    $data = $request->post($url, $this->request()->getSwooleRequest()->getContent(), ['timeout' => 3, 'headers' => $header, 'cookies' => $cookies]);
                    $responseHeaders = $data->getHeaders();
                    if ($responseHeaders['server-uu']) {
                        $this->header('server-uu', current($responseHeaders['server-uu']));
                    }
                    $this->header('content-type', 'application/octet-stream');
                    $content=$data->getBody()->getContents();
//                    if ($this->request()->getQueryParam('ct')){
//                        $encode=new Encoder();
//                        $encode->decode($content);
//                        $content=$encode->encode($content);
//                    }
                    $this->response()->getSwooleResponse()->write($content);
                } catch (\Throwable $exception) {
                    $errorCode = $exception->getCode();
                    switch ($errorCode) {
                        case 403:
                            $this->setStatusCode(403);
                            $this->response()->getSwooleResponse()->write($exception->getMessage());
                            break;
                        default:
                            $this->setStatusCode(403);
                            $this->header('cc', $exception->getCode());
                            $this->response()->getSwooleResponse()->write($exception->getMessage());

                    }
                }
                break;
        }
    }
}